import React, { useEffect, useRef } from "react";
import * as THREE from "three";
import { STLLoader } from "three/examples/jsm/loaders/STLLoader";
import { OrbitControls } from "three/examples/jsm/controls/OrbitControls";
import { autoScale, getTriangleVertexs } from "./util";

export const STLViewer: React.FC = () => {
  const containerRef = useRef<HTMLDivElement>(null);

  useEffect(() => {
    if (containerRef.current) {
      const width = containerRef.current.clientWidth;
      const height = containerRef.current.clientHeight;

      // 创建场景、相机和渲染器
      const scene = new THREE.Scene();
      const camera = new THREE.PerspectiveCamera(75, width / height, 0.1, 1000);
      camera.position.z = 5;

      const renderer = new THREE.WebGLRenderer({ antialias: true });
      renderer.setSize(width, height);
      containerRef.current.appendChild(renderer.domElement);

      // 加载STL文件
      const loader = new STLLoader();
      loader.load("/1.stl", (geometry) => {
        const s = autoScale(geometry);
        // geometry.center();
        // const material = new THREE.MeshPhongMaterial({
        //   color: "lightgrey", // 使用更亮的颜色
        //   shininess: 100, // 增加亮度
        //   specular: 0x222222, // 调整镜面高光的颜色
        //   // side: THREE.DoubleSide,
        //   // wireframe: true,
        // });
        // const mesh = new THREE.Mesh(geometry, material);
        // mesh.scale.set(scale, scale, scale);
        // scene.add(mesh);

        // 输出三角形顶点信息
        const triangles = getTriangleVertexs(geometry);
        ////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////

        const newTriangles = [];

        triangles.forEach((triangleVertices) => {
          const A = triangleVertices[0];
          const B = triangleVertices[1];
          const C = triangleVertices[2];

          // 计算三角形的法线
          let normal = new THREE.Vector3();
          let cb = new THREE.Vector3();
          let ab = new THREE.Vector3();
          cb.subVectors(C, B);
          ab.subVectors(A, B);
          cb.cross(ab).normalize();
          normal.copy(cb);

          // 创建新的顶点，它们沿着法线方向内移2个单位
          let A1 = new THREE.Vector3()
            .copy(A)
            .add(normal.clone().multiplyScalar(-2));
          let B1 = new THREE.Vector3()
            .copy(B)
            .add(normal.clone().multiplyScalar(-2));
          let C1 = new THREE.Vector3()
            .copy(C)
            .add(normal.clone().multiplyScalar(-2));

          // 添加新的三角形
          newTriangles.push([A1, B1, C1]);

          // 创建封闭表面的新面片
          newTriangles.push([A, B, A1]);
          newTriangles.push([B, B1, A1]);

          newTriangles.push([B, C, B1]);
          newTriangles.push([C, C1, B1]);

          newTriangles.push([C, A, C1]);
          newTriangles.push([A, A1, C1]);
        });

        const g = new THREE.BufferGeometry();

        // 将newTriangles数据分解为顶点和索引数组
        const vertices = [];
        const indices = [];
        newTriangles.forEach((triangle, index) => {
          triangle.forEach((vertex) => {
            vertices.push(vertex.x, vertex.y, vertex.z);
          });
          indices.push(index * 3, index * 3 + 1, index * 3 + 2);
        });

        // 设置vertices和indices到geometry
        g.setAttribute(
          "position",
          new THREE.Float32BufferAttribute(vertices, 3)
        );
        g.setIndex(indices);

        // 为了确保光照和阴影正确，我们需要计算几何体的面的法线
        g.computeVertexNormals();
        g.center();

        // 创建网格使用MeshPhongMaterial或其他的材料
        const material = new THREE.MeshPhongMaterial({
          color: 0x00ff00,
          side: THREE.DoubleSide,
        });

        const mesh = new THREE.Mesh(g, material);
        mesh.scale.set(s, s, s);

        // 添加网格到场景
        scene.add(mesh);

        ////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////
      });

      // 添加环境光源
      const ambientLight = new THREE.AmbientLight(0xffffff, 1); // 第二个参数为强度
      scene.add(ambientLight);

      // 添加方向光源
      const directionalLight = new THREE.DirectionalLight(0xffffff, 2);
      directionalLight.position.set(1, 1, 1).normalize();
      scene.add(directionalLight);

      // 添加XYZ轴
      const axesHelper = new THREE.AxesHelper(2);
      scene.add(axesHelper);

      // 添加OrbitControls
      const controls = new OrbitControls(camera, renderer.domElement);

      // 渲染函数
      const animate = () => {
        requestAnimationFrame(animate);
        renderer.render(scene, camera);
      };

      animate();
    }
  }, []);

  return <div ref={containerRef} style={{ width: "100vw", height: "100vh" }} />;
};
